@@ -2,7 +2,7 @@
 BORIS
 Behavioral Observation Research Interactive Software
 
-Copyright 2012-2020 Olivier Friard
+Copyright 2012-2022 Olivier Friard
 
 This file is part of BORIS.
 
@@ -21,6 +21,6 @@ This file is part of BORIS.
 
 """
 
-from .boris import main
+from .core import main
 
 name = "BORIS"
