@@ -20,19 +20,47 @@ This file is part of BORIS.
 
 """
 
-from PyQt5.QtGui import *
-from PyQt5.QtCore import *
-from PyQt5.QtWidgets import *
-
-import decimal
-from decimal import getcontext
-import json
 import binascii
-import os
 import io
+import json
+import os
+from decimal import ROUND_DOWN
+from decimal import Decimal as dec
+from decimal import getcontext
 
-from boris.config import *
-from boris import dialog
+from PyQt5.QtCore import QBuffer, QByteArray, QIODevice, QLineF, QPoint, Qt, pyqtSignal
+from PyQt5.QtGui import QBrush, QColor, QIcon, QMouseEvent, QPen, QPixmap, QPolygonF
+from PyQt5.QtWidgets import (
+    QAction,
+    QApplication,
+    QColorDialog,
+    QFileDialog,
+    QFrame,
+    QGraphicsEllipseItem,
+    QGraphicsLineItem,
+    QGraphicsPixmapItem,
+    QGraphicsPolygonItem,
+    QGraphicsScene,
+    QGraphicsView,
+    QHBoxLayout,
+    QInputDialog,
+    QLabel,
+    QLineEdit,
+    QListWidget,
+    QMainWindow,
+    QMessageBox,
+    QPushButton,
+    QSizePolicy,
+    QSlider,
+    QSpacerItem,
+    QSplitter,
+    QVBoxLayout,
+    QWidget,
+)
+
+from . import config as cfg
+from . import dialog
+from . import utilities as util
 
 designColor = QColor(255, 0, 0, 128)  # red opacity: 50%
 penWidth = 0
@@ -42,55 +70,6 @@ selectedBrush.setStyle(Qt.SolidPattern)
 selectedBrush.setColor(QColor(255, 255, 0, 255))
 
 
-def intersection(A, B, C, D):
-    """
-    line segments intersection with decimal precision
-    return True when intersection else False
-    """
-    getcontext().prec = 28
-
-    Dec = decimal.Decimal
-    xa, ya = Dec(str(A[0])), Dec(str(A[1]))
-    xb, yb = Dec(str(B[0])), Dec(str(B[1]))
-    xc, yc = Dec(str(C[0])), Dec(str(C[1]))
-    xd, yd = Dec(str(D[0])), Dec(str(D[1]))
-
-    # check if first segment is vertical
-    try:
-        if xa == xb:
-            slope = (yc - yd) / (xc - xd)
-            intersept = yc - slope * xc
-            xm = xa
-            ym = slope * xm + intersept
-
-        # check if second segment is vertical
-        elif xc == xd:
-            slope = (ya - yb) / (xa - xb)
-            intersept = ya - slope * xa
-            xm = xc
-            ym = slope * xm + intersept
-        else:
-            xm = ((xd * xa * yc - xd * xb * yc - xd * xa * yb - xc * xa * yd + xc * xa * yb + xd * ya * xb +
-                   xc * xb * yd - xc * ya * xb) /
-                  (-yb * xd + yb * xc + ya * xd - ya * xc + xb * yd - xb * yc - xa * yd + xa * yc)).quantize(
-                      Dec('.001'), rounding=decimal.ROUND_DOWN)
-            ym = ((yb * xc * yd - yb * yc * xd - ya * xc * yd + ya * yc * xd - xa * yb * yd + xa * yb * yc +
-                   ya * xb * yd - ya * xb * yc) /
-                  (-yb * xd + yb * xc + ya * xd - ya * xc + xb * yd - xb * yc - xa * yd + xa * yc)).quantize(
-                      Dec('.001'), rounding=decimal.ROUND_DOWN)
-
-        xmin1, xmax1 = min(xa, xb), max(xa, xb)
-        xmin2, xmax2 = min(xc, xd), max(xc, xd)
-        ymin1, ymax1 = min(ya, yb), max(ya, yb)
-        ymin2, ymax2 = min(yc, yd), max(yc, yd)
-
-        return (xm >= xmin1 and xm <= xmax1 and xm >= xmin2 and xm <= xmax2 and ym >= ymin1 and ym <= ymax1 and
-                ym >= ymin2 and ym <= ymax2)
-
-    except Exception:  # for cases xa=xb=xc=xd
-        return True
-
-
 class BehaviorsMapCreatorWindow(QMainWindow):
 
     signal_add_to_project = pyqtSignal(dict)
@@ -99,6 +78,7 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         """
         class for handling mousepress event in QGraphicsView
         """
+
         mousePress = pyqtSignal(QMouseEvent)
 
         def mousePressEvent(self, event):
@@ -114,7 +94,7 @@ class BehaviorsMapCreatorWindow(QMainWindow):
             self.scene().update()
 
     bitmapFileName, mapName, fileName = "", "", ""
-    flagNewArea, flagMapChanged = False, False
+    flagNewArea, flag_map_changed = False, False
     polygonsList2 = []
     areaColor = QColor("lime")
 
@@ -180,11 +160,11 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         fileMenu.addAction(self.exitAction)
 
         splitter1 = QSplitter(Qt.Vertical)
-        '''
+        """
         splitter1.addWidget(splitter1)
         splitter1.addWidget(bottom)
-        '''
-        '''vlayout_list = QVBoxLayout()'''
+        """
+        """vlayout_list = QVBoxLayout()"""
 
         self.view = self.View(self)
         self.view.mousePress.connect(self.viewMousePressEvent)
@@ -246,8 +226,7 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         self.btEditAreaCode.setVisible(False)
         hlayout_area.addWidget(self.btEditAreaCode)
 
-        self.btColor = QPushButton()
-        self.btColor.clicked.connect(self.chooseColor)
+        self.btColor = QPushButton(clicked=self.chooseColor)
         self.btColor.setVisible(False)
         self.btColor.setStyleSheet(f"QWidget {{background-color:{self.areaColor.name()}}}")
         hlayout_area.addWidget(self.btColor)
@@ -274,13 +253,13 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         frame.setLayout(vlayout_frame)
 
         vlayout = QVBoxLayout()
-        '''
+        """
         vlayout.addWidget(self.view)
         vlayout.addWidget(QLabel("Defined area"))
         vlayout.addWidget(self.area_list)
-        '''
+        """
         vlayout.addWidget(splitter1)
-        '''vlayout.addLayout(vlayout_view_list)'''
+        """vlayout.addLayout(vlayout_view_list)"""
 
         vlayout.addWidget(frame)
         vlayout.addWidget(self.btLoad)
@@ -296,8 +275,21 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         add coding map to project
         """
 
-        mapDict = self.make_coding_map_dict()
-        self.signal_add_to_project.emit(mapDict)
+        map_dict = self.make_coding_map_dict()
+        if map_dict["areas"] == {}:
+
+            QMessageBox.critical(
+                self,
+                cfg.programName,
+                (
+                    "The behaviors coding map does not contain any behavior area.<br>"
+                    "Add some behavior areas before adding the coding map to the project"
+                ),
+            )
+
+            return
+
+        self.signal_add_to_project.emit(map_dict)
 
     def area_list_item_click(self, item):
         """
@@ -342,8 +334,9 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         else:
             code_index = 0
 
-        item, ok = QInputDialog.getItem(self, "Select a behavior", "Available behaviors", self.codes_list, code_index,
-                                        False)
+        item, ok = QInputDialog.getItem(
+            self, "Select a behavior", "Available behaviors", self.codes_list, code_index, False
+        )
         self.leAreaCode.setText(item)
 
         if self.selectedPolygon:
@@ -380,11 +373,14 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         area color button clicked
         """
         cd = QColorDialog()
+        cd.setWindowFlags(Qt.WindowStaysOnTopHint)
+        cd.setOptions(QColorDialog.DontUseNativeDialog)
 
-        col = cd.getColor()
-        if col.isValid():
-            self.btColor.setStyleSheet("QWidget {background-color:%s}" % col.name())
-            self.areaColor = col
+        # col = cd.getColor()
+        # if col.isValid():
+        if cd.exec_():
+            self.areaColor = cd.currentColor()
+            self.btColor.setStyleSheet(f"QWidget {{background-color:{self.areaColor.name()}}}")
             self.areaColor.setAlpha(int(self.slAlpha.value() / 100 * 255))
 
         if self.selectedPolygon:
@@ -402,20 +398,23 @@ class BehaviorsMapCreatorWindow(QMainWindow):
 
     def closeEvent(self, event):
 
-        if self.flagMapChanged:
+        if self.flag_map_changed:
 
-            response = dialog.MessageDialog("BORIS - Modifiers map creator",
-                                            "What to do about the current unsaved modifiers coding map?",
-                                            ["Save", "Discard", "Cancel"])
+            response = dialog.MessageDialog(
+                "BORIS - Behaviors map creator",
+                "What to do about the current unsaved behaviors coding map?",
+                [cfg.SAVE, cfg.DISCARD, cfg.CANCEL],
+            )
 
-            if response == "Save":
+            if response == cfg.SAVE:
                 if not self.saveMap_clicked():
                     event.ignore()
 
-            if response == "Cancel":
+            if response == cfg.CANCEL:
                 event.ignore()
                 return
 
+        self.flag_map_changed = False
         event.accept()
 
     def viewMousePressEvent(self, event):
@@ -423,13 +422,29 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         check if area selected with mouse
         """
 
+        def add_polygon():
+            # create polygon
+            newPolygon = QPolygonF()
+            for p in self.view.points:
+                newPolygon.append(QPoint(p[0], p[1]))
+
+            # draw polygon a red polygon
+            self.closedPolygon = QGraphicsPolygonItem(newPolygon)
+            self.closedPolygon.setPen(QPen(designColor, penWidth, penStyle, Qt.RoundCap, Qt.RoundJoin))
+            self.closedPolygon.setBrush(self.areaColor)
+            self.view.scene().addItem(self.closedPolygon)
+
         if not self.bitmapFileName:
             return
 
         test = self.view.mapToScene(event.pos()).toPoint()  # coordinates of clicked point
 
-        if test.x() < 0 or test.y() < 0 or test.x() > self.pixmap.size().width() or test.y() > self.pixmap.size(
-        ).height():
+        if (
+            test.x() < 0
+            or test.y() < 0
+            or test.x() > self.pixmap.size().width()
+            or test.y() > self.pixmap.size().height()
+        ):
             return
 
         if not self.flagNewArea:  # test clicked point for areas
@@ -448,8 +463,9 @@ class BehaviorsMapCreatorWindow(QMainWindow):
 
                     self.selectedPolygonMemBrush = self.selectedPolygon.brush()
 
-                    self.selectedPolygon.setPen(QPen(QColor(255, 0, 0, 255), 2, Qt.SolidLine, Qt.RoundCap,
-                                                     Qt.RoundJoin))
+                    self.selectedPolygon.setPen(
+                        QPen(QColor(255, 0, 0, 255), 2, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin)
+                    )
 
                     self.lb.setVisible(True)
                     self.leAreaCode.setText(areaCode)
@@ -461,7 +477,8 @@ class BehaviorsMapCreatorWindow(QMainWindow):
 
                     self.areaColor = self.selectedPolygon.brush().color()
                     self.btColor.setStyleSheet(
-                        f"QWidget {{background-color:{self.selectedPolygon.brush().color().name()}}}")
+                        f"QWidget {{background-color:{self.selectedPolygon.brush().color().name()}}}"
+                    )
                     self.btColor.setVisible(True)
 
                     self.slAlpha.setValue(int(self.selectedPolygon.brush().color().alpha() / 255 * 100))
@@ -504,10 +521,14 @@ class BehaviorsMapCreatorWindow(QMainWindow):
                 # test is polygon is crossed
                 if len(self.view.points) >= 3:
 
-                    for idx, point in enumerate(self.view.points[:-2]):
+                    for idx, _ in enumerate(self.view.points[:-2]):
 
-                        if intersection(self.view.points[idx], self.view.points[idx + 1], self.view.points[-1],
-                                        (int(end.x()), int(end.y()))):
+                        if util.intersection(
+                            self.view.points[idx],
+                            self.view.points[idx + 1],
+                            self.view.points[-1],
+                            (int(end.x()), int(end.y())),
+                        ):
                             QMessageBox.critical(self, "", "The polygon edges can not be intersected")
                             return
 
@@ -515,7 +536,8 @@ class BehaviorsMapCreatorWindow(QMainWindow):
                 if abs(end.x() - self.view.points[0][0]) < 10 and abs(end.y() - self.view.points[0][1]) < 10:
 
                     line = QGraphicsLineItem(
-                        QLineF(self.view._start, QPoint(self.view.points[0][0], self.view.points[0][1])))
+                        QLineF(self.view._start, QPoint(self.view.points[0][0], self.view.points[0][1]))
+                    )
                     line.setPen(QPen(designColor, penWidth, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
 
                     self.view.scene().addItem(line)
@@ -524,27 +546,13 @@ class BehaviorsMapCreatorWindow(QMainWindow):
                     self.statusBar().showMessage("Area completed")
 
                     # create polygon
-                    newPolygon = QPolygonF()
-                    for p in self.view.points:
-                        newPolygon.append(QPoint(p[0], p[1]))
-
-                    # draw polygon a red polygon
-
-                    self.closedPolygon = QGraphicsPolygonItem(newPolygon)
-
-                    self.closedPolygon.setPen(QPen(designColor, penWidth, penStyle, Qt.RoundCap, Qt.RoundJoin))
-                    self.closedPolygon.setBrush(self.areaColor)
-
-                    self.view.scene().addItem(self.closedPolygon)
-
+                    add_polygon()
                     return
 
                 self.view.points.append((int(end.x()), int(end.y())))
 
                 line = QGraphicsLineItem(QLineF(self.view._start, end))
-
                 line.setPen(QPen(designColor, 2, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
-
                 self.view.scene().addItem(line)
                 self.view.elList.append(line)
 
@@ -567,22 +575,59 @@ class BehaviorsMapCreatorWindow(QMainWindow):
 
                 self.view.points.append((self.view._start.x(), self.view._start.y()))
 
+        # automatically close the polygon
+        if event.buttons() == Qt.MiddleButton and not self.closedPolygon:
+
+            # add first point as last point of polygon
+
+            # test is polygon is crossed
+
+            if len(self.view.points) >= 3:
+
+                for idx, _ in enumerate(self.view.points[1:-2]):
+
+                    if util.intersection(
+                        self.view.points[idx],
+                        self.view.points[idx + 1],
+                        self.view.points[-1],
+                        self.view.points[0],
+                    ):
+                        QMessageBox.critical(self, "", "The polygon edges can not be intersected")
+                        return
+
+            line = QGraphicsLineItem(
+                QLineF(self.view.points[-1][0], self.view.points[-1][1], self.view.points[0][0], self.view.points[0][1])
+            )
+            line.setPen(QPen(designColor, 2, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+            self.view.scene().addItem(line)
+            self.view.elList.append(line)
+
+            self.view.points.append(self.view.points[0])
+            add_polygon()
+            self.statusBar().showMessage("Area completed")
+
     def mapName_clicked(self):
         """
         Edit map name
         """
 
         while True:
-            map_name, ok = QInputDialog.getText(self, "Behaviors coding map name", "Enter a name for the coding map",
-                                                QLineEdit.Normal, self.mapName)
+            map_name, ok = QInputDialog.getText(
+                self, "Behaviors coding map name", "Enter a name for the coding map", QLineEdit.Normal, self.mapName
+            )
             if map_name.upper() in self.bcm_list:
                 QMessageBox.critical(
-                    self, "", ("The name for the new coding map already exists.<br>"
-                               f"{', '.join(self.bcm_list)} are already defined.<br>"
-                               "To reuse the same name the existing coding map must be deleted (File > Edit project)"))
+                    self,
+                    "",
+                    (
+                        "The name for the new coding map already exists.<br>"
+                        f"{', '.join(self.bcm_list)} are already defined.<br>"
+                        "To reuse the same name the existing coding map must be deleted (File > Edit project)"
+                    ),
+                )
             if ok and map_name and map_name.upper() not in self.bcm_list:
                 self.mapName = map_name
-                self.setWindowTitle(f"{programName} - Behaviors coding map creator - {self.mapName}")
+                self.setWindowTitle(f"{cfg.programName} - Behaviors coding map creator - {self.mapName}")
                 break
             if not ok:
                 return
@@ -592,40 +637,49 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         create a new map
         """
 
-        if self.flagMapChanged:
+        if self.flag_map_changed:
 
-            response = dialog.MessageDialog(programName + " - Behaviors coding map creator",
-                                            "What to do about the current unsaved coding map?", [SAVE, DISCARD, CANCEL])
+            response = dialog.MessageDialog(
+                cfg.programName + " - Behaviors coding map creator",
+                "What to do about the current unsaved coding map?",
+                [cfg.SAVE, cfg.DISCARD, cfg.CANCEL],
+            )
 
-            if response == SAVE:
+            if response == cfg.SAVE:
                 if not self.saveMap_clicked():
                     return
 
-            if response == CANCEL:
+            if response == cfg.CANCEL:
                 return
 
         self.cancelMap()
 
         while True:
-            map_name, ok = QInputDialog.getText(self, "Behaviors coding map name",
-                                                "Enter a name for the new coding map")
+            map_name, ok = QInputDialog.getText(
+                self, "Behaviors coding map name", "Enter a name for the new coding map"
+            )
             if map_name.upper() in self.bcm_list:
                 QMessageBox.critical(
-                    self, "", ("The name for the new coding map already exists.<br>"
-                               f"{', '.join(self.bcm_list)} are already defined.<br>"
-                               "To reuse the same name the existing coding map must be deleted (File > Edit project)"))
+                    self,
+                    "",
+                    (
+                        "The name for the new coding map already exists.<br>"
+                        f"{', '.join(self.bcm_list)} are already defined.<br>"
+                        "To reuse the same name the existing coding map must be deleted (File > Edit project)"
+                    ),
+                )
             if ok and map_name and map_name.upper() not in self.bcm_list:
                 self.mapName = map_name
                 break
             if not ok:
                 return
-        '''
+        """
         if not self.mapName:
             QMessageBox.critical(self, "", "You must define a name for the new coding map")
             return
-        '''
+        """
 
-        self.setWindowTitle(f"{programName} - Behaviors coding map creator tool - {self.mapName}")
+        self.setWindowTitle(f"{cfg.programName} - Behaviors coding map creator tool - {self.mapName}")
 
         self.btLoad.setVisible(True)
         self.statusBar().showMessage('Click "Load bitmap" button to select and load a bitmap into the viewer')
@@ -637,16 +691,19 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         load bitmap from data
         show it in view scene
         """
-        if self.flagMapChanged:
-            response = dialog.MessageDialog(programName + " - Behaviors coding map creator",
-                                            "What to do about the current unsaved coding map?",
-                                            ['Save', 'Discard', 'Cancel'])
+        if self.flag_map_changed:
+            response = dialog.MessageDialog(
+                cfg.programName + " - Behaviors coding map creator",
+                "What to do about the current unsaved coding map?",
+                ["Save", "Discard", "Cancel"],
+            )
 
             if (response == "Save" and not self.saveMap_clicked()) or (response == "Cancel"):
                 return
 
-        fn = QFileDialog(self).getOpenFileName(self, "Open a behaviors coding map", "",
-                                               "Behaviors coding map (*.behav_coding_map);;All files (*)")
+        fn = QFileDialog(self).getOpenFileName(
+            self, "Open a behaviors coding map", "", "Behaviors coding map (*.behav_coding_map);;All files (*)"
+        )
         fileName = fn[0] if type(fn) is tuple else fn
 
         if fileName:
@@ -654,18 +711,20 @@ class BehaviorsMapCreatorWindow(QMainWindow):
             try:
                 self.codingMap = json.loads(open(fileName, "r").read())
             except Exception:
-                QMessageBox.critical(self, programName, f"The file {fileName} is not a behaviors coding map.")
+                QMessageBox.critical(self, cfg.programName, f"The file {fileName} is not a behaviors coding map.")
                 return
 
-            if "coding_map_type" not in self.codingMap or self.codingMap[
-                    "coding_map_type"] != "BORIS behaviors coding map":
-                QMessageBox.critical(self, programName, f"The file {fileName} is not a BORIS behaviors coding map.")
+            if (
+                "coding_map_type" not in self.codingMap
+                or self.codingMap["coding_map_type"] != "BORIS behaviors coding map"
+            ):
+                QMessageBox.critical(self, cfg.programName, f"The file {fileName} is not a BORIS behaviors coding map.")
 
             self.cancelMap()
 
             self.mapName = self.codingMap["name"]
 
-            self.setWindowTitle(f"{programName} - Behaviors coding map creator - {self.mapName}")
+            self.setWindowTitle(f"{cfg.programName} - Behaviors coding map creator - {self.mapName}")
 
             self.bitmapFileName = True
 
@@ -691,7 +750,7 @@ class BehaviorsMapCreatorWindow(QMainWindow):
                     newPolygon.append(QPoint(p[0], p[1]))
 
                 # draw polygon
-                '''polygon = QGraphicsPolygonItem(None, None) if QT_VERSION_STR[0] == "4" else QGraphicsPolygonItem()'''
+                """polygon = QGraphicsPolygonItem(None, None) if QT_VERSION_STR[0] == "4" else QGraphicsPolygonItem()"""
                 polygon = QGraphicsPolygonItem()
                 polygon.setPolygon(newPolygon)
                 clr = QColor()
@@ -716,22 +775,26 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         else:
             self.statusBar().showMessage("No file", 5000)
 
-    def make_coding_map_dict(self):
-        mapDict = {"coding_map_type": "BORIS behaviors coding map", "name": self.mapName, "areas": {}}
+    def make_coding_map_dict(self) -> dict:
+        """
+        create the coding map dictionary
+        the image is encoded in bse64 format
+        """
+        map_dict = {"coding_map_type": "BORIS behaviors coding map", "name": self.mapName, "areas": {}}
 
         for ac, pg in self.polygonsList2:
-            if not mapDict["areas"]:
+            if not map_dict["areas"]:
                 idx = 0
             else:
-                idx = max(mapDict["areas"].keys()) + 1
+                idx = max(map_dict["areas"].keys()) + 1
 
             points = []
             for p in range(pg.polygon().count()):
                 points.append([int(pg.polygon().value(p).x()), int(pg.polygon().value(p).y())])
 
-            mapDict["areas"][idx] = {"code": ac, "geometry": points, "color": pg.brush().color().rgba()}
+            map_dict["areas"][idx] = {"code": ac, "geometry": points, "color": pg.brush().color().rgba()}
 
-        mapDict["areas"] = json.loads(json.dumps(mapDict["areas"]))
+        map_dict["areas"] = json.loads(json.dumps(map_dict["areas"]))
 
         # Save QPixmap to QByteArray via QBuffer.
         byte_array = QByteArray()
@@ -742,9 +805,9 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         string_io.seek(0)
 
         # add codified bitmap
-        mapDict["bitmap"] = binascii.b2a_base64(string_io.read()).decode("utf-8")
+        map_dict["bitmap"] = binascii.b2a_base64(string_io.read()).decode("utf-8")
 
-        return mapDict
+        return map_dict
 
     def saveMap(self):
         """
@@ -757,7 +820,7 @@ class BehaviorsMapCreatorWindow(QMainWindow):
             with open(self.fileName, "w") as outfile:
                 outfile.write(json.dumps(mapDict))
 
-            self.flagMapChanged = False
+            self.flag_map_changed = False
 
             return True
         else:
@@ -778,8 +841,12 @@ class BehaviorsMapCreatorWindow(QMainWindow):
     def saveMap_clicked(self):
 
         if not self.fileName:
-            fn = QFileDialog().getSaveFileName(self, "Save modifiers map", self.mapName + ".behav_coding_map",
-                                               "Behaviors coding map (*.behav_coding_map);;All files (*)")
+            fn = QFileDialog().getSaveFileName(
+                self,
+                "Save modifiers map",
+                self.mapName + ".behav_coding_map",
+                "Behaviors coding map (*.behav_coding_map);;All files (*)",
+            )
             self.fileName = fn[0] if type(fn) is tuple else fn
 
             if self.fileName and os.path.splitext(self.fileName)[1] != ".behav_coding_map":
@@ -791,9 +858,8 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         return False
 
     def newArea(self):
-
         if not self.bitmapFileName:
-            QMessageBox.critical(self, programName, "A bitmap must be loaded before to define areas")
+            QMessageBox.critical(self, cfg.programName, "A bitmap must be loaded before to define areas")
             return
 
         if self.selectedPolygon:
@@ -813,22 +879,29 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         self.slAlpha.setVisible(True)
         self.btDeleteArea.setVisible(False)
 
-        self.statusBar().showMessage(("Click on bitmap to set the vertices of the area with the mouse "
-                                      "(right click will cancel the last point)"))
+        self.statusBar().showMessage(
+            (
+                "Click on bitmap to set the vertices of the area with the mouse "
+                "(right click will cancel the last point)"
+            )
+        )
 
     def saveArea(self):
 
         if not self.closedPolygon:
-            QMessageBox.critical(self, programName, ("You must close your area before saving it.\n"
-                                                     "The last vertex must correspond to the first one."))
+            QMessageBox.critical(
+                self,
+                cfg.programName,
+                ("You must close your area before saving it.\n" "The last vertex must correspond to the first one."),
+            )
 
         if len(self.view.points) < 3:
-            QMessageBox.critical(self, programName, "You must define a closed area")
+            QMessageBox.critical(self, cfg.programName, "You must define a closed area")
             return
 
         # check if no area code
         if not self.leAreaCode.text():
-            QMessageBox.critical(self, programName, "You must define a code for the new behavior area")
+            QMessageBox.critical(self, cfg.programName, "You must define a code for the new behavior area")
             return
 
         # remove all lines
@@ -845,8 +918,15 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         self.view.points, self.view.elList = [], []
 
         for widget in [
-                self.btSaveArea, self.btCancelAreaCreation, self.lb, self.leAreaCode, self.btEditAreaCode, self.btColor,
-                self.slAlpha, self.btDeleteArea, self.btNewArea
+            self.btSaveArea,
+            self.btCancelAreaCreation,
+            self.lb,
+            self.leAreaCode,
+            self.btEditAreaCode,
+            self.btColor,
+            self.slAlpha,
+            self.btDeleteArea,
+            self.btNewArea,
         ]:
             widget.setVisible(False)
 
@@ -854,7 +934,7 @@ class BehaviorsMapCreatorWindow(QMainWindow):
 
         self.leAreaCode.setText("")
 
-        self.flagMapChanged = True
+        self.flag_map_changed = True
         self.statusBar().showMessage("New area saved", 5000)
 
         self.update_area_list()
@@ -910,7 +990,7 @@ class BehaviorsMapCreatorWindow(QMainWindow):
             if to_delete != -1:
                 del self.polygonsList2[to_delete]
 
-            self.flagMapChanged = True
+            self.flag_map_changed = True
 
         self.view.elList = []
 
@@ -949,7 +1029,7 @@ class BehaviorsMapCreatorWindow(QMainWindow):
         self.mapNameAction.setEnabled(False)
         self.statusBar().showMessage("")
 
-        self.flagMapChanged = False
+        self.flag_map_changed = False
 
     def loadBitmap(self):
         """
@@ -966,12 +1046,19 @@ class BehaviorsMapCreatorWindow(QMainWindow):
             self.pixmap.load(self.bitmapFileName)
 
             # scale image
-            if self.pixmap.size().width() > CODING_MAP_RESIZE_W or self.pixmap.size().height() > CODING_MAP_RESIZE_H:
-                self.pixmap = self.pixmap.scaled(CODING_MAP_RESIZE_W, CODING_MAP_RESIZE_H, Qt.KeepAspectRatio)
+            if (
+                self.pixmap.size().width() > cfg.CODING_MAP_RESIZE_W
+                or self.pixmap.size().height() > cfg.CODING_MAP_RESIZE_H
+            ):
+                self.pixmap = self.pixmap.scaled(cfg.CODING_MAP_RESIZE_W, cfg.CODING_MAP_RESIZE_H, Qt.KeepAspectRatio)
                 QMessageBox.information(
-                    self, programName,
-                    (f"The bitmap was resized to {self.pixmap.size().width()}x{self.pixmap.size().height()} pixels\n"
-                     "The original file was not modified"))
+                    self,
+                    cfg.programName,
+                    (
+                        f"The bitmap was resized to {self.pixmap.size().width()}x{self.pixmap.size().height()} pixels\n"
+                        "The original file was not modified"
+                    ),
+                )
 
             self.view.setSceneRect(0, 0, self.pixmap.size().width(), self.pixmap.size().height())
             pixitem = QGraphicsPixmapItem(self.pixmap)
@@ -988,14 +1075,37 @@ class BehaviorsMapCreatorWindow(QMainWindow):
 
             self.statusBar().showMessage("""Click "New behavior area" to create a new behavior area""")
 
-            self.flagMapChanged = True
+            self.flag_map_changed = True
 
 
-if __name__ == '__main__':
+def behaviors_coding_map_creator(self):
+    """
+    show behaviors coding map creator window
+    """
+
+    if not self.project:
+        QMessageBox.warning(
+            self, cfg.programName, "No project found", QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton
+        )
+        return
+
+    codes_list = [self.pj[cfg.ETHOGRAM][key][cfg.BEHAVIOR_CODE] for key in self.pj[cfg.ETHOGRAM]]
+
+    self.mapCreatorWindow = BehaviorsMapCreatorWindow(codes_list)
+    # behaviors coding map list
+    self.mapCreatorWindow.bcm_list = [x["name"].upper() for x in self.pj.get(cfg.BEHAVIORS_CODING_MAP, [])]
+    self.mapCreatorWindow.signal_add_to_project.connect(self.behaviors_coding_map_creator_signal_addtoproject)
+    self.mapCreatorWindow.move(self.pos())
+    self.mapCreatorWindow.resize(cfg.CODING_MAP_RESIZE_W, cfg.CODING_MAP_RESIZE_H)
+    self.mapCreatorWindow.show()
+
+
+if __name__ == "__main__":
 
     import sys
+
     app = QApplication(sys.argv)
     window = BehaviorsMapCreatorWindow(["North zone", "East zone", "South zone", "West zone"])
-    window.resize(CODING_MAP_RESIZE_W, CODING_MAP_RESIZE_H)
+    window.resize(cfg.CODING_MAP_RESIZE_W, cfg.CODING_MAP_RESIZE_H)
     window.show()
     sys.exit(app.exec_())
