@@ -21,24 +21,23 @@ This file is part of BORIS.
 """
 
 import logging
-import os
-import sys
 
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import QDialog
 
-from boris import duration_widget
-from boris.config import *
-from boris.param_panel_ui import Ui_Dialog
+from . import config as cfg
+from . import duration_widget
+from .param_panel_ui import Ui_Dialog
 
 
 class Param_panel(QDialog, Ui_Dialog):
-
     def __init__(self, parent=None):
 
         super().__init__()
         self.setupUi(self)
 
+        self.media_duration = None
+
         # insert duration widget for time offset
         self.start_time = duration_widget.Duration_widget(0)
         self.horizontalLayout.insertWidget(1, self.start_time)
@@ -58,15 +57,34 @@ class Param_panel(QDialog, Ui_Dialog):
 
         self.lwBehaviors.itemClicked.connect(self.behavior_item_clicked)
 
-        self.rb_full.clicked.connect(lambda: self.rb_time(TIME_FULL_OBS))
-        self.rb_limit.clicked.connect(lambda: self.rb_time(TIME_EVENTS))
-        self.rb_interval.clicked.connect(lambda: self.rb_time(TIME_ARBITRARY_INTERVAL))
+        self.rb_observed_events.setChecked(True)
 
-    def rb_time(self, button):
+        self.rb_media_duration.clicked.connect(lambda: self.rb_time_interval_selection(cfg.TIME_FULL_OBS))
+        self.rb_observed_events.clicked.connect(lambda: self.rb_time_interval_selection(cfg.TIME_EVENTS))
+        self.rb_user_defined.clicked.connect(lambda: self.rb_time_interval_selection(cfg.TIME_ARBITRARY_INTERVAL))
+
+    def rb_time_interval_selection(self, button):
         """
-        time
+        select the time interval for operation
         """
-        self.frm_time_interval.setEnabled(button == TIME_ARBITRARY_INTERVAL)
+        if button == cfg.TIME_ARBITRARY_INTERVAL:
+            self.frm_time_interval.setEnabled(True)
+            self.frm_time_interval.setVisible(True)
+
+        elif button == cfg.TIME_EVENTS and len(self.selectedObservations) == 1:
+            self.start_time.set_time(self.start_coding)
+            self.end_time.set_time(self.end_coding)
+            self.frm_time_interval.setEnabled(False)
+            self.frm_time_interval.setVisible(True)
+
+        elif button == cfg.TIME_FULL_OBS and len(self.selectedObservations) == 1 and self.media_duration is not None:
+            self.start_time.set_time(0)
+            self.end_time.set_time(self.media_duration)
+            self.frm_time_interval.setEnabled(False)
+            self.frm_time_interval.setVisible(True)
+
+        else:
+            self.frm_time_interval.setVisible(False)
 
     def subjects_button_clicked(self, command):
         for idx in range(self.lwSubjects.count()):
@@ -136,33 +154,33 @@ class Param_panel(QDialog, Ui_Dialog):
         observed_behaviors = []
 
         # extract events from selected observations
-        all_events = [self.pj[OBSERVATIONS][x][EVENTS] for x in self.pj[OBSERVATIONS] if x in selected_observations]
+        all_events = [
+            self.pj[cfg.OBSERVATIONS][x][cfg.EVENTS] for x in self.pj[cfg.OBSERVATIONS] if x in selected_observations
+        ]
 
         for events in all_events:
             for event in events:
-                if (event[EVENT_SUBJECT_FIELD_IDX] in selected_subjects or
-                    (not event[EVENT_SUBJECT_FIELD_IDX] and NO_FOCAL_SUBJECT in selected_subjects)):
-                    observed_behaviors.append(event[EVENT_BEHAVIOR_FIELD_IDX])
+                if event[cfg.EVENT_SUBJECT_FIELD_IDX] in selected_subjects or (
+                    not event[cfg.EVENT_SUBJECT_FIELD_IDX] and cfg.NO_FOCAL_SUBJECT in selected_subjects
+                ):
+                    observed_behaviors.append(event[cfg.EVENT_BEHAVIOR_FIELD_IDX])
 
         # remove duplicate
         return list(set(observed_behaviors))
 
     def cb_changed(self):
-        selectedSubjects = []
+        selected_subjects: list = []
         for idx in range(self.lwSubjects.count()):
             cb = self.lwSubjects.itemWidget(self.lwSubjects.item(idx))
             if cb and cb.isChecked():
-                selectedSubjects.append(cb.text())
+                selected_subjects.append(cb.text())
 
-        # FIX ME
-        observedBehaviors = self.extract_observed_behaviors(self.selectedObservations, selectedSubjects)
-
-        logging.debug(f"observed behaviors: {observedBehaviors}")
+        observed_behaviors = self.extract_observed_behaviors(self.selectedObservations, selected_subjects)
 
         for idx in range(self.lwBehaviors.count()):
 
             if self.lwBehaviors.item(idx).data(33) != "category":
-                if self.lwBehaviors.item(idx).text() in observedBehaviors:
+                if self.lwBehaviors.item(idx).text() in observed_behaviors:
                     self.lwBehaviors.item(idx).setCheckState(Qt.Checked)
                 else:
                     self.lwBehaviors.item(idx).setCheckState(Qt.Unchecked)
