@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 """
 BORIS
 Behavioral Observation Research Interactive Software
@@ -22,10 +21,7 @@ This file is part of BORIS.
 """
 
 import datetime as dt
-import json
-import logging
-import pathlib
-import sys
+import pathlib as pl
 
 import matplotlib
 
@@ -48,12 +44,9 @@ from matplotlib.dates import (
     rrulewrapper,
 )
 
-from boris import db_functions
-from boris import project_functions
-from boris import utilities
-from boris.config import *
-
-plt_colors = dict(mcolors.BASE_COLORS, **mcolors.CSS4_COLORS)
+from . import config as cfg
+from . import db_functions, project_functions, observation_operations
+from . import utilities as util
 
 
 def default_value(ethogram, behavior, parameter):
@@ -112,7 +105,7 @@ def create_behaviors_bar_plot(
     param: dict,
     plot_directory: str,
     output_format: str,
-    plot_colors: list = BEHAVIORS_PLOT_COLORS,
+    plot_colors: list = cfg.BEHAVIORS_PLOT_COLORS,
 ) -> dict:
     """
     time budget bar plot
@@ -127,11 +120,11 @@ def create_behaviors_bar_plot(
         dict:
     """
 
-    selected_subjects = param[SELECTED_SUBJECTS]
-    selected_behaviors = param[SELECTED_BEHAVIORS]
+    selected_subjects = param[cfg.SELECTED_SUBJECTS]
+    selected_behaviors = param[cfg.SELECTED_BEHAVIORS]
     time_interval = param["time"]
-    start_time = param[START_TIME]
-    end_time = param[END_TIME]
+    start_time = param[cfg.START_TIME]
+    end_time = param[cfg.END_TIME]
 
     parameters = ["duration", "number of occurences"]
 
@@ -141,114 +134,129 @@ def create_behaviors_bar_plot(
 
     if not ok:
         return {"error": True, "message": msg}
-    try:
 
-        # extract all behaviors from ethogram for colors in plot
-        all_behaviors = [pj[ETHOGRAM][x][BEHAVIOR_CODE] for x in utilities.sorted_keys(pj[ETHOGRAM])]
+    # extract all behaviors from ethogram for colors in plot
+    all_behaviors = util.all_behaviors(pj[cfg.ETHOGRAM])
 
-        for obs_id in selected_observations:
+    for obs_id in selected_observations:
 
-            cursor = db_connector.cursor()
-            # distinct behaviors
-            cursor.execute("SELECT distinct behavior FROM aggregated_events WHERE observation = ?", (obs_id,))
-            distinct_behav = [rows["behavior"] for rows in cursor.fetchall()]
+        cursor = db_connector.cursor()
+        # distinct behaviors
+        cursor.execute("SELECT distinct behavior FROM aggregated_events WHERE observation = ?", (obs_id,))
+        distinct_behav = [rows["behavior"] for rows in cursor.fetchall()]
 
-            # add selected behaviors that are not observed
-            """
-            if not param[EXCLUDE_BEHAVIORS]:
-                for behavior in selected_behaviors:
-                    if [x for x in distinct_behav if x == behavior] == []:
-                        distinct_behav.append(behavior)
-            """
+        # add selected behaviors that are not observed
+        """
+        if not param[EXCLUDE_BEHAVIORS]:
+            for behavior in selected_behaviors:
+                if [x for x in distinct_behav if x == behavior] == []:
+                    distinct_behav.append(behavior)
+        """
 
-            # distinct subjects
-            cursor.execute("SELECT distinct subject FROM aggregated_events WHERE observation = ?", (obs_id,))
-            distinct_subjects = [rows["subject"] for rows in cursor.fetchall()]
+        # distinct subjects
+        cursor.execute("SELECT distinct subject FROM aggregated_events WHERE observation = ?", (obs_id,))
+        distinct_subjects = [rows["subject"] for rows in cursor.fetchall()]
 
-            behaviors = init_behav(pj[ETHOGRAM], distinct_subjects, distinct_behav, parameters)
+        behaviors = init_behav(pj[cfg.ETHOGRAM], distinct_subjects, distinct_behav, parameters)
 
-            # plot creation
-            if len(distinct_subjects) > 1:
-                fig, axs = plt.subplots(nrows=1, ncols=len(distinct_subjects), sharey=True)
-                fig2, axs2 = plt.subplots(nrows=1, ncols=len(distinct_subjects), sharey=True)
+        # plot creation
+        if len(distinct_subjects) > 1:
+            fig, axs = plt.subplots(nrows=1, ncols=len(distinct_subjects), sharey=True)
+            fig2, axs2 = plt.subplots(nrows=1, ncols=len(distinct_subjects), sharey=True)
 
-            else:
-                fig, ax = plt.subplots(nrows=1, ncols=len(distinct_subjects), sharey=True)
-                axs = np.ndarray(shape=(1), dtype=type(ax))
-                axs[0] = ax
+        else:
+            fig, ax = plt.subplots(nrows=1, ncols=len(distinct_subjects), sharey=True)
+            axs = np.ndarray(shape=(1), dtype=type(ax))
+            axs[0] = ax
 
-                fig2, ax2 = plt.subplots(nrows=1, ncols=len(distinct_subjects), sharey=True)
-                axs2 = np.ndarray(shape=(1), dtype=type(ax2))
-                axs2[0] = ax2
+            fig2, ax2 = plt.subplots(nrows=1, ncols=len(distinct_subjects), sharey=True)
+            axs2 = np.ndarray(shape=(1), dtype=type(ax2))
+            axs2[0] = ax2
 
-            fig.suptitle("Durations of behaviors")
-            fig2.suptitle("Number of occurences of behaviors")
+        fig.suptitle("Durations of behaviors")
+        fig2.suptitle("Number of occurences of behaviors")
 
-            # if modifiers not to be included set modifiers to ""
-            cursor.execute("UPDATE aggregated_events SET modifiers = ''")
+        # if modifiers not to be included set modifiers to ""
+        cursor.execute("UPDATE aggregated_events SET modifiers = ''")
 
-            # time
-            obs_length = project_functions.observation_total_length(pj[OBSERVATIONS][obs_id])
-            if obs_length == -1:
-                obs_length = 0
+        # time
+        obs_length = observation_operations.observation_total_length(pj[cfg.OBSERVATIONS][obs_id])
+        if obs_length == -1:
+            obs_length = 0
 
-            if param["time"] == TIME_FULL_OBS:
+        if param["time"] == cfg.TIME_FULL_OBS:
+            min_time = float(0)
+            max_time = float(obs_length)
+
+        if param["time"] == cfg.TIME_EVENTS:
+            try:
+                min_time = float(pj[cfg.OBSERVATIONS][obs_id][cfg.EVENTS][0][0])
+            except Exception:
                 min_time = float(0)
+            try:
+                max_time = float(pj[cfg.OBSERVATIONS][obs_id][cfg.EVENTS][-1][0])
+            except Exception:
                 max_time = float(obs_length)
 
-            if param["time"] == TIME_EVENTS:
-                try:
-                    min_time = float(pj[OBSERVATIONS][obs_id][EVENTS][0][0])
-                except Exception:
-                    min_time = float(0)
-                try:
-                    max_time = float(pj[OBSERVATIONS][obs_id][EVENTS][-1][0])
-                except Exception:
-                    max_time = float(obs_length)
+        if param["time"] == cfg.TIME_ARBITRARY_INTERVAL:
+            min_time = float(start_time)
+            max_time = float(end_time)
 
-            if param["time"] == TIME_ARBITRARY_INTERVAL:
-                min_time = float(start_time)
-                max_time = float(end_time)
+        cursor.execute(
+            "UPDATE aggregated_events SET start = ? WHERE observation = ? AND start < ? AND stop BETWEEN ? AND ?",
+            (
+                min_time,
+                obs_id,
+                min_time,
+                min_time,
+                max_time,
+            ),
+        )
+        cursor.execute(
+            "UPDATE aggregated_events SET stop = ? WHERE observation = ? AND stop > ? AND start BETWEEN ? AND ?",
+            (
+                max_time,
+                obs_id,
+                max_time,
+                min_time,
+                max_time,
+            ),
+        )
+        cursor.execute(
+            "UPDATE aggregated_events SET start = ?, stop = ? WHERE observation = ? AND start < ? AND stop > ?",
+            (
+                min_time,
+                max_time,
+                obs_id,
+                min_time,
+                max_time,
+            ),
+        )
 
-            cursor.execute(
-                "UPDATE aggregated_events SET start = ? WHERE observation = ? AND start < ? AND stop BETWEEN ? AND ?",
-                (
-                    min_time,
-                    obs_id,
-                    min_time,
-                    min_time,
-                    max_time,
-                ),
-            )
-            cursor.execute(
-                "UPDATE aggregated_events SET stop = ? WHERE observation = ? AND stop > ? AND start BETWEEN ? AND ?",
-                (
-                    max_time,
-                    obs_id,
-                    max_time,
-                    min_time,
-                    max_time,
-                ),
-            )
-            cursor.execute(
-                "UPDATE aggregated_events SET start = ?, stop = ? WHERE observation = ? AND start < ? AND stop > ?",
-                (
-                    min_time,
-                    max_time,
-                    obs_id,
-                    min_time,
-                    max_time,
-                ),
-            )
+        for ax_idx, subject in enumerate(sorted(distinct_subjects)):
 
-            for ax_idx, subject in enumerate(sorted(distinct_subjects)):
+            for behavior in distinct_behav:
 
-                for behavior in distinct_behav:
+                # number of occurences
+                cursor.execute(
+                    (
+                        "SELECT COUNT(*) AS count FROM aggregated_events "
+                        "WHERE observation = ? AND subject = ? AND behavior = ?"
+                    ),
+                    (
+                        obs_id,
+                        subject,
+                        behavior,
+                    ),
+                )
+                for row in cursor.fetchall():
+                    behaviors[subject][behavior]["number of occurences"] = 0 if row["count"] is None else row["count"]
 
-                    # number of occurences
+                # total duration
+                if cfg.STATE in project_functions.event_type(behavior, pj[cfg.ETHOGRAM]):
                     cursor.execute(
                         (
-                            "SELECT COUNT(*) AS count FROM aggregated_events "
+                            "SELECT SUM(stop - start) AS duration FROM aggregated_events "
                             "WHERE observation = ? AND subject = ? AND behavior = ?"
                         ),
                         (
@@ -258,119 +266,94 @@ def create_behaviors_bar_plot(
                         ),
                     )
                     for row in cursor.fetchall():
-                        behaviors[subject][behavior]["number of occurences"] = (
-                            0 if row["count"] is None else row["count"]
-                        )
+                        behaviors[subject][behavior]["duration"] = 0 if row["duration"] is None else row["duration"]
 
-                    # total duration
-                    if STATE in project_functions.event_type(behavior, pj[ETHOGRAM]):
-                        cursor.execute(
-                            (
-                                "SELECT SUM(stop - start) AS duration FROM aggregated_events "
-                                "WHERE observation = ? AND subject = ? AND behavior = ?"
-                            ),
-                            (
-                                obs_id,
-                                subject,
-                                behavior,
-                            ),
-                        )
-                        for row in cursor.fetchall():
-                            behaviors[subject][behavior]["duration"] = 0 if row["duration"] is None else row["duration"]
+            durations, n_occurences, colors, x_labels, colors_duration, x_labels_duration = [], [], [], [], [], []
 
-                durations, n_occurences, colors, x_labels, colors_duration, x_labels_duration = [], [], [], [], [], []
+            for behavior in sorted(distinct_behav):
 
-                for behavior in sorted(distinct_behav):
+                if param[cfg.EXCLUDE_BEHAVIORS] and behaviors[subject][behavior]["number of occurences"] == 0:
+                    continue
 
-                    if param[EXCLUDE_BEHAVIORS] and behaviors[subject][behavior]["number of occurences"] == 0:
-                        continue
+                n_occurences.append(behaviors[subject][behavior]["number of occurences"])
+                x_labels.append(behavior)
+                try:
+                    colors.append(util.behavior_color(plot_colors, all_behaviors.index(behavior)))
+                except Exception:
+                    colors.append("darkgray")
 
-                    n_occurences.append(behaviors[subject][behavior]["number of occurences"])
-                    x_labels.append(behavior)
+                if cfg.STATE in project_functions.event_type(behavior, pj[cfg.ETHOGRAM]):
+                    durations.append(behaviors[subject][behavior]["duration"])
+                    x_labels_duration.append(behavior)
                     try:
-                        colors.append(utilities.behavior_color(plot_colors, all_behaviors.index(behavior)))
+                        colors_duration.append(util.behavior_color(plot_colors, all_behaviors.index(behavior)))
                     except Exception:
-                        colors.append("darkgray")
+                        colors_duration.append("darkgray")
 
-                    if STATE in project_functions.event_type(behavior, pj[ETHOGRAM]):
-                        durations.append(behaviors[subject][behavior]["duration"])
-                        x_labels_duration.append(behavior)
-                        try:
-                            colors_duration.append(utilities.behavior_color(plot_colors, all_behaviors.index(behavior)))
-                        except Exception:
-                            colors_duration.append("darkgray")
+            # width = 0.35       # the width of the bars: can also be len(x) sequence
 
-                # width = 0.35       # the width of the bars: can also be len(x) sequence
+            axs2[ax_idx].bar(
+                np.arange(len(n_occurences)),
+                n_occurences,
+                # width,
+                color=colors,
+            )
 
-                axs2[ax_idx].bar(
-                    np.arange(len(n_occurences)),
-                    n_occurences,
-                    # width,
-                    color=colors,
-                )
+            axs[ax_idx].bar(
+                np.arange(len(durations)),
+                durations,
+                # width,
+                color=colors_duration,
+            )
 
-                axs[ax_idx].bar(
-                    np.arange(len(durations)),
-                    durations,
-                    # width,
-                    color=colors_duration,
-                )
+            if ax_idx == 0:
+                axs[ax_idx].set_ylabel("Duration (s)")
+            axs[ax_idx].set_xlabel("Behaviors")
+            axs[ax_idx].set_title(f"{subject}")
 
-                if ax_idx == 0:
-                    axs[ax_idx].set_ylabel("Duration (s)")
-                axs[ax_idx].set_xlabel("Behaviors")
-                axs[ax_idx].set_title(f"{subject}")
+            axs[ax_idx].set_xticks(np.arange(len(durations)))
+            axs[ax_idx].set_xticklabels(x_labels_duration, rotation="vertical", fontsize=8)
 
-                axs[ax_idx].set_xticks(np.arange(len(durations)))
-                axs[ax_idx].set_xticklabels(x_labels_duration, rotation="vertical", fontsize=8)
+            if ax_idx == 0:
+                axs2[ax_idx].set_ylabel("Number of occurences")
+            axs2[ax_idx].set_xlabel("Behaviors")
+            axs2[ax_idx].set_title(f"{subject}")
 
-                if ax_idx == 0:
-                    axs2[ax_idx].set_ylabel("Number of occurences")
-                axs2[ax_idx].set_xlabel("Behaviors")
-                axs2[ax_idx].set_title(f"{subject}")
+            axs2[ax_idx].set_xticks(np.arange(len(n_occurences)))
+            axs2[ax_idx].set_xticklabels(x_labels, rotation="vertical", fontsize=8)
 
-                axs2[ax_idx].set_xticks(np.arange(len(n_occurences)))
-                axs2[ax_idx].set_xticklabels(x_labels, rotation="vertical", fontsize=8)
+        fig.align_labels()
+        fig.tight_layout(rect=[0, 0.03, 1, 0.95])
 
-            fig.align_labels()
-            fig.tight_layout(rect=[0, 0.03, 1, 0.95])
+        fig2.align_labels()
+        fig2.tight_layout(rect=[0, 0.03, 1, 0.95])
 
-            fig2.align_labels()
-            fig2.tight_layout(rect=[0, 0.03, 1, 0.95])
+        if plot_directory:
+            # output_file_name = f"{pathlib.Path(plot_directory) / utilities.safeFileName(obs_id)}.{output_format}"
+            fig.savefig(f"{pl.Path(plot_directory) / util.safeFileName(obs_id)}.duration.{output_format}")
+            fig2.savefig(f"{pl.Path(plot_directory) / util.safeFileName(obs_id)}.number_of_occurences.{output_format}")
+            plt.close()
+        else:
+            fig.show()
+            fig2.show()
 
-            if plot_directory:
-                # output_file_name = f"{pathlib.Path(plot_directory) / utilities.safeFileName(obs_id)}.{output_format}"
-                fig.savefig(f"{pathlib.Path(plot_directory) / utilities.safeFileName(obs_id)}.duration.{output_format}")
-                fig2.savefig(
-                    f"{pathlib.Path(plot_directory) / utilities.safeFileName(obs_id)}.number_of_occurences.{output_format}"
-                )
-                plt.close()
-            else:
-                fig.show()
-                fig2.show()
-
-        return {}
-
-    except Exception:
-        error_type, error_file_name, error_lineno = utilities.error_info(sys.exc_info())
-        logging.critical(f"Error in time budget bar plot: {error_type} {error_file_name} {error_lineno}")
-        return {"error": True, "exception": sys.exc_info()}
+    return {}
 
 
 def create_events_plot(
-    pj, selected_observations, parameters, plot_colors=BEHAVIORS_PLOT_COLORS, plot_directory="", file_format="png"
+    pj, selected_observations, parameters, plot_colors=cfg.BEHAVIORS_PLOT_COLORS, plot_directory="", file_format="png"
 ):
     """
     create a time diagram plot (sort of gantt chart)
     with matplotlib barh function (https://matplotlib.org/3.1.0/api/_as_gen/matplotlib.pyplot.barh.html)
     """
 
-    selected_subjects = parameters[SELECTED_SUBJECTS]
-    selected_behaviors = parameters[SELECTED_BEHAVIORS]
-    include_modifiers = parameters[INCLUDE_MODIFIERS]
-    interval = parameters[TIME_INTERVAL]
-    start_time = parameters[START_TIME]
-    end_time = parameters[END_TIME]
+    selected_subjects = parameters[cfg.SELECTED_SUBJECTS]
+    selected_behaviors = parameters[cfg.SELECTED_BEHAVIORS]
+    include_modifiers = parameters[cfg.INCLUDE_MODIFIERS]
+    interval = parameters[cfg.TIME_INTERVAL]
+    start_time = parameters[cfg.START_TIME]
+    end_time = parameters[cfg.END_TIME]
 
     ok, msg, db_connector = db_functions.load_aggregated_events_in_db(
         pj, selected_subjects, selected_observations, selected_behaviors
@@ -396,7 +379,7 @@ def create_events_plot(
     distinct_behav_modif = sorted(distinct_behav_modif)
     max_len = len(distinct_behav_modif)
 
-    all_behaviors = [pj[ETHOGRAM][x][BEHAVIOR_CODE] for x in utilities.sorted_keys(pj[ETHOGRAM])]
+    all_behaviors = [pj[cfg.ETHOGRAM][x][cfg.BEHAVIOR_CODE] for x in util.sorted_keys(pj[cfg.ETHOGRAM])]
 
     par1 = 1
     bar_height = 0.5
@@ -434,34 +417,33 @@ def create_events_plot(
         max_len = len(distinct_behav_modif)
 
         # time
-        obs_length = project_functions.observation_total_length(pj[OBSERVATIONS][obs_id])
+        obs_length = observation_operations.observation_total_length(pj[cfg.OBSERVATIONS][obs_id])
         if obs_length == -1:  # media length not available
-            interval = TIME_EVENTS
+            interval = cfg.TIME_EVENTS
 
-        if interval == TIME_FULL_OBS:
-            min_time = float(0)
+        if interval == cfg.TIME_FULL_OBS:
+            min_time = 0.0
             max_time = float(obs_length)
 
-        if interval == TIME_EVENTS:
+        if interval == cfg.TIME_EVENTS:
             try:
-                min_time = float(pj[OBSERVATIONS][obs_id][EVENTS][0][0])
+                min_time = float(pj[cfg.OBSERVATIONS][obs_id][cfg.EVENTS][0][0])  # first event
             except Exception:
-                min_time = float(0)
+                min_time = 0.0
             try:
-                max_time = float(pj[OBSERVATIONS][obs_id][EVENTS][-1][0])
+                max_time = float(pj[cfg.OBSERVATIONS][obs_id][cfg.EVENTS][-1][0])  # last event
             except Exception:
                 max_time = float(obs_length)
 
-        if interval == TIME_ARBITRARY_INTERVAL:
+        if interval == cfg.TIME_ARBITRARY_INTERVAL:
             min_time = float(start_time)
             max_time = float(end_time)
 
         # adjust start if start < init
         cursor.execute(
-            "UPDATE aggregated_events SET start = ? WHERE observation = ? AND start < ? AND stop BETWEEN ? AND ?",
+            "UPDATE aggregated_events SET start = ? WHERE  start < ? AND stop BETWEEN ? AND ?",
             (
                 min_time,
-                obs_id,
                 min_time,
                 min_time,
                 max_time,
@@ -469,10 +451,9 @@ def create_events_plot(
         )
         # adjust stop if stop > end
         cursor.execute(
-            "UPDATE aggregated_events SET stop = ? WHERE observation = ? AND stop > ? AND start BETWEEN ? AND ?",
+            "UPDATE aggregated_events SET stop = ? WHERE stop > ? AND start BETWEEN ? AND ?",
             (
                 max_time,
-                obs_id,
                 max_time,
                 min_time,
                 max_time,
@@ -480,11 +461,10 @@ def create_events_plot(
         )
         # adjust start and stop if start < init and stop > end
         cursor.execute(
-            "UPDATE aggregated_events SET start = ?, stop = ? WHERE observation = ? AND start < ? AND stop > ?",
+            "UPDATE aggregated_events SET start = ?, stop = ? WHERE start < ? AND stop > ?",
             (
                 min_time,
                 max_time,
-                obs_id,
                 min_time,
                 max_time,
             ),
@@ -501,7 +481,7 @@ def create_events_plot(
 
             if parameters["exclude behaviors"]:
                 cursor.execute(
-                    "SELECT distinct behavior, modifiers FROM aggregated_events WHERE subject = ?", (subject,)
+                    "SELECT DISTINCT behavior, modifiers FROM aggregated_events WHERE subject = ?", (subject,)
                 )
                 distinct_behav_modif = [[rows["behavior"], rows["modifiers"]] for rows in cursor.fetchall()]
 
@@ -530,10 +510,9 @@ def create_events_plot(
                 cursor.execute(
                     (
                         "SELECT start, stop FROM aggregated_events "
-                        "WHERE observation = ? AND subject = ? AND behavior = ? AND modifiers = ?"
+                        "WHERE  subject = ? AND behavior = ? AND modifiers = ?"
                     ),
                     (
-                        obs_id,
                         subject,
                         behavior,
                         modifiers,
@@ -545,13 +524,15 @@ def create_events_plot(
                     start_date = matplotlib.dates.date2num(init + dt.timedelta(seconds=row["start"]))
                     end_date = matplotlib.dates.date2num(
                         init
-                        + dt.timedelta(seconds=row["stop"] + POINT_EVENT_PLOT_DURATION * (row["stop"] == row["start"]))
+                        + dt.timedelta(
+                            seconds=row["stop"] + cfg.POINT_EVENT_PLOT_DURATION * (row["stop"] == row["start"])
+                        )
                     )
                     try:
-                        bar_color = utilities.behavior_color(plot_colors, all_behaviors.index(behavior))
+                        bar_color = util.behavior_color(plot_colors, all_behaviors.index(behavior))
                     except Exception:
                         bar_color = "darkgray"
-                    bar_color = POINT_EVENT_PLOT_COLOR if row["stop"] == row["start"] else bar_color
+                    bar_color = cfg.POINT_EVENT_PLOT_COLOR if row["stop"] == row["start"] else bar_color
 
                     # sage colors removed from matplotlib colors list
                     if bar_color in ["sage", "darksage", "lightsage"]:
@@ -611,6 +592,6 @@ def create_events_plot(
         plt.tight_layout()
 
         if len(selected_observations) > 1:
-            plt.savefig(f"{pathlib.Path(plot_directory) / utilities.safeFileName(obs_id)}.{file_format}")
+            plt.savefig(f"{pl.Path(plot_directory) / util.safeFileName(obs_id)}.{file_format}")
         else:
             plt.show()
