@@ -27,10 +27,12 @@ import matplotlib
 
 matplotlib.use("Qt5Agg")
 import numpy as np
-from PyQt5.QtWidgets import *
+from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QPushButton, QLabel, QSpinBox)
 from PyQt5.QtCore import pyqtSignal, QEvent
+from PyQt5 import Qt
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
 from matplotlib.figure import Figure
+import matplotlib.ticker as mticker
 
 
 class Plot_spectrogram_RT(QWidget):
@@ -38,22 +40,9 @@ class Plot_spectrogram_RT(QWidget):
     # send keypress event to mainwindow
     sendEvent = pyqtSignal(QEvent)
 
-    def get_wav_info(self, wav_file: str):
-        """
-        read wav file and extract information
-        """
-        try:
-            wav = wave.open(wav_file, "r")
-            frames = wav.readframes(-1)
-            sound_info = np.fromstring(frames, dtype=np.int16)
-            frame_rate = wav.getframerate()
-            wav.close()
-            return sound_info, frame_rate
-        except Exception:
-            return np.array([]), 0
-
     def __init__(self):
         super().__init__()
+        self.setWindowTitle(f"Spectrogram")
 
         self.interval = 10  # interval of visualization (in seconds)
         self.time_mem = -1
@@ -63,6 +52,7 @@ class Plot_spectrogram_RT(QWidget):
         self.spectro_color_map = matplotlib.pyplot.get_cmap("viridis")
 
         self.figure = Figure()
+        self.ax = self.figure.add_subplot(1, 1, 1)
 
         self.canvas = FigureCanvas(self.figure)
 
@@ -71,24 +61,20 @@ class Plot_spectrogram_RT(QWidget):
 
         hlayout1 = QHBoxLayout()
         hlayout1.addWidget(QLabel("Time interval"))
-        button_time_inc = QPushButton("+", self)
-        button_time_inc.clicked.connect(lambda: self.time_interval_changed(1))
-        button_time_dec = QPushButton("-", self)
-        button_time_dec.clicked.connect(lambda: self.time_interval_changed(-1))
-        hlayout1.addWidget(button_time_inc)
-        hlayout1.addWidget(button_time_dec)
+        hlayout1.addWidget(
+            QPushButton("+", self, clicked=lambda: self.time_interval_changed(1), focusPolicy=Qt.Qt.NoFocus))
+        hlayout1.addWidget(
+            QPushButton("-", self, clicked=lambda: self.time_interval_changed(-1), focusPolicy=Qt.Qt.NoFocus))
         layout.addLayout(hlayout1)
 
         hlayout2 = QHBoxLayout()
         hlayout2.addWidget(QLabel("Frequency interval"))
-        self.sb_freq_min = QSpinBox()
+        self.sb_freq_min = QSpinBox(valueChanged=self.frequency_interval_changed, focusPolicy=Qt.Qt.NoFocus)
         self.sb_freq_min.setRange(0, 200000)
         self.sb_freq_min.setSingleStep(100)
-        self.sb_freq_min.valueChanged.connect(self.frequency_interval_changed)
-        self.sb_freq_max = QSpinBox()
+        self.sb_freq_max = QSpinBox(valueChanged=self.frequency_interval_changed, focusPolicy=Qt.Qt.NoFocus)
         self.sb_freq_max.setRange(0, 200000)
         self.sb_freq_max.setSingleStep(100)
-        self.sb_freq_max.valueChanged.connect(self.frequency_interval_changed)
         hlayout2.addWidget(self.sb_freq_min)
         hlayout2.addWidget(self.sb_freq_max)
         layout.addLayout(hlayout2)
@@ -101,22 +87,49 @@ class Plot_spectrogram_RT(QWidget):
         """
         send event (if keypress) to main window
         """
-        if event.type() == QEvent.KeyPress:
+        if (event.type() == QEvent.KeyPress):
             self.sendEvent.emit(event)
             return True
         else:
             return False
 
+    def get_wav_info(self, wav_file: str):
+        """
+        read wav file and extract information
+
+        Args:
+            wav_file (str): path of wav file
+
+        Returns:
+            np.array: signal contained in wav file
+            int: frame rate of wav file
+
+        """
+
+        try:
+            wav = wave.open(wav_file, "r")
+            frames = wav.readframes(-1)
+            sound_info = np.fromstring(frames, dtype=np.int16)
+            frame_rate = wav.getframerate()
+            wav.close()
+            return sound_info, frame_rate
+        except Exception:
+            return np.array([]), 0
+
     def time_interval_changed(self, action: int):
         """
         change the time interval for plotting spectrogram
 
         Args:
             action (int): -1 decrease time interval, +1 increase time interval
+
+        Returns:
+            None
         """
+
         if action == -1 and self.interval <= 5:
             return
-        self.interval += 5 * action
+        self.interval += (5 * action)
         self.plot_spectro(current_time=self.time_mem, force_plot=True)
 
     def frequency_interval_changed(self):
@@ -138,8 +151,6 @@ class Plot_spectrogram_RT(QWidget):
 
         try:
             self.sound_info, self.frame_rate = self.get_wav_info(wav_file_path)
-            print(f"{self.sound_info=}")
-            print(f"{self.frame_rate=}")
             if not self.frame_rate:
                 return {"error": f"unknown format for file {wav_file_path}"}
         except FileNotFoundError:
@@ -157,6 +168,7 @@ class Plot_spectrogram_RT(QWidget):
 
         Args:
             current_time (float): time for displaying spectrogram
+            force_plot (bool): force plot even if media paused
         """
 
         if not force_plot and current_time == self.time_mem:
@@ -164,76 +176,68 @@ class Plot_spectrogram_RT(QWidget):
 
         self.time_mem = current_time
 
-        ax = self.figure.add_subplot(1, 1, 1)
-
-        ax.clear()
+        self.ax.clear()
 
         # start
         if current_time <= self.interval / 2:
 
-            Pxx, freqs, bins, im = ax.specgram(
-                self.sound_info[: int((self.interval) * self.frame_rate)],
+            self.ax.specgram(
+                self.sound_info[:int((self.interval) * self.frame_rate)],
                 mode="psd",
-                # NFFT=1024,
+                #NFFT=1024,
                 Fs=self.frame_rate,
-                # noverlap=900,
-                cmap=self.spectro_color_map,
-            )
+                #noverlap=900,
+                cmap=self.spectro_color_map)
 
-            ax.set_xlim(current_time - self.interval / 2, current_time + self.interval / 2)
+            self.ax.set_xlim(current_time - self.interval / 2, current_time + self.interval / 2)
 
             # cursor
-            ax.axvline(x=current_time, color=self.cursor_color, linestyle="-")
+            self.ax.axvline(x=current_time, color=self.cursor_color, linestyle="-")
 
         elif current_time >= self.media_length - self.interval / 2:
 
             i = int(round(len(self.sound_info) - (self.interval * self.frame_rate), 0))
 
-            Pxx, freqs, bins, im = ax.specgram(
+            self.ax.specgram(
                 self.sound_info[i:],
                 mode="psd",
-                # NFFT=1024,
+                #NFFT=1024,
                 Fs=self.frame_rate,
-                # noverlap=900,
-                cmap=self.spectro_color_map,
-            )
+                #noverlap=900,
+                cmap=self.spectro_color_map)
 
             lim1 = current_time - (self.media_length - self.interval / 2)
             lim2 = lim1 + self.interval
 
-            ax.set_xlim(lim1, lim2)
+            self.ax.set_xlim(lim1, lim2)
 
-            ax.set_xticklabels([str(round(w + self.media_length - self.interval, 1)) for w in ax.get_xticks()])
+            self.ax.xaxis.set_major_locator(mticker.FixedLocator(self.ax.get_xticks().tolist()))
+            self.ax.set_xticklabels(
+                [str(round(w + self.media_length - self.interval, 1)) for w in self.ax.get_xticks()])
 
             # cursor
-            ax.axvline(x=lim1 + self.interval / 2, color=self.cursor_color, linestyle="-")
+            self.ax.axvline(x=lim1 + self.interval / 2, color=self.cursor_color, linestyle="-")
 
         # middle
         else:
 
-            start = (current_time - self.interval / 2) * self.frame_rate
-            end = (current_time + self.interval / 2) * self.frame_rate
-
-            Pxx, freqs, bins, im = ax.specgram(
-                self.sound_info[
-                    int(round((current_time - self.interval / 2) * self.frame_rate, 0)) : int(
-                        round((current_time + self.interval / 2) * self.frame_rate, 0)
-                    )
-                ],
+            self.ax.specgram(
+                self.sound_info[int(round(
+                    (current_time - self.interval / 2) *
+                    self.frame_rate, 0)):int(round((current_time + self.interval / 2) * self.frame_rate, 0))],
                 mode="psd",
-                # NFFT=1024,
+                #NFFT=1024,
                 Fs=self.frame_rate,
-                # noverlap=900,
-                cmap=self.spectro_color_map,
-            )
+                #noverlap=900,
+                cmap=self.spectro_color_map)
 
-            ax.set_xticklabels([str(round(current_time + w - self.interval / 2, 1)) for w in ax.get_xticks()])
+            self.ax.xaxis.set_major_locator(mticker.FixedLocator(self.ax.get_xticks().tolist()))
+            self.ax.set_xticklabels([str(round(current_time + w - self.interval / 2, 1)) for w in self.ax.get_xticks()])
 
             # cursor
-            ax.axvline(x=self.interval / 2, color=self.cursor_color, linestyle="-")
+            self.ax.axvline(x=self.interval / 2, color=self.cursor_color, linestyle="-")
 
-        ax.set_ylim(self.sb_freq_min.value(), self.sb_freq_max.value())
-
-        self.figure.subplots_adjust(wspace=0, hspace=0)
+        self.ax.set_ylim(self.sb_freq_min.value(), self.sb_freq_max.value())
+        '''self.figure.subplots_adjust(wspace=0, hspace=0)'''
 
         self.canvas.draw()
