@@ -27,14 +27,12 @@ import matplotlib
 
 matplotlib.use("Qt5Agg")
 import numpy as np
-from PyQt5.QtWidgets import *
+from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QPushButton, QLabel)
 from PyQt5.QtCore import pyqtSignal, QEvent
+from PyQt5 import Qt
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
 from matplotlib.figure import Figure
-
-Y_MIN = 0
-Y_MAX = 20000
-VMIN = -40
+import matplotlib.ticker as mticker
 
 
 class Plot_waveform_RT(QWidget):
@@ -42,34 +40,33 @@ class Plot_waveform_RT(QWidget):
     # send keypress event to mainwindow
     sendEvent = pyqtSignal(QEvent)
 
-    def get_wav_info(self, wav_file: str):
-        """
-        read wav file and extract information
-        """
-        try:
-            wav = wave.open(wav_file, "r")
-            frames = wav.readframes(-1)
-            signal = np.fromstring(frames, dtype=np.int16)
-            frame_rate = wav.getframerate()
-            wav.close()
-            return signal, frame_rate
-        except Exception:
-            return np.array([]), 0
-
     def __init__(self):
         super().__init__()
+        self.setWindowTitle(f"Waveform")
 
         self.interval = 60  # interval of visualization (in seconds)
         self.time_mem = -1
 
         self.cursor_color = "red"
 
+        self.spectro_color_map = matplotlib.pyplot.get_cmap("viridis")
+
         self.figure = Figure()
+        self.ax = self.figure.add_subplot(1, 1, 1)
 
         self.canvas = FigureCanvas(self.figure)
-        layout = QVBoxLayout()
 
+        layout = QVBoxLayout()
         layout.addWidget(self.canvas)
+
+        hlayout1 = QHBoxLayout()
+        hlayout1.addWidget(QLabel("Time interval"))
+        hlayout1.addWidget(
+            QPushButton("+", self, clicked=lambda: self.time_interval_changed(1), focusPolicy=Qt.Qt.NoFocus))
+        hlayout1.addWidget(
+            QPushButton("-", self, clicked=lambda: self.time_interval_changed(-1), focusPolicy=Qt.Qt.NoFocus))
+        layout.addLayout(hlayout1)
+
         self.setLayout(layout)
 
         self.installEventFilter(self)
@@ -84,6 +81,28 @@ class Plot_waveform_RT(QWidget):
         else:
             return False
 
+    def get_wav_info(self, wav_file: str):
+        """
+        read wav file and extract information
+
+        Args:
+            wav_file (str): path of wav file
+
+        Returns:
+            np.array: signal contained in wav file
+            int: frame rate of wav file
+
+        """
+        try:
+            wav = wave.open(wav_file, "r")
+            frames = wav.readframes(-1)
+            signal = np.fromstring(frames, dtype=np.int16)
+            frame_rate = wav.getframerate()
+            wav.close()
+            return signal, frame_rate
+        except Exception:
+            return np.array([]), 0
+
     def load_wav(self, wav_file_path: str) -> dict:
         """
         load wav file in numpy array
@@ -107,108 +126,87 @@ class Plot_waveform_RT(QWidget):
 
         return {"media_length": self.media_length, "frame_rate": self.frame_rate}
 
-    def plot_waveform(self, current_time: float):
+    def time_interval_changed(self, action: int) -> None:
+        """
+        change the time interval for plotting waveform
+
+        Args:
+            action (int): -1 decrease time interval, +1 increase time interval
+
+        Returns:
+            None
+        """
+
+        if action == -1 and self.interval <= 5:
+            return
+        self.interval += (5 * action)
+        self.plot_waveform(current_time=self.time_mem, force_plot=True)
+
+    def plot_waveform(self, current_time: float, force_plot: bool = False):
         """
         plot sound waveform centered on the current time
 
         Args:
             current_time (float): time for displaying waveform
+            force_plot (bool): force plot even if media paused
         """
 
-        if current_time == self.time_mem:
+        if not force_plot and current_time == self.time_mem:
             return
 
         self.time_mem = current_time
 
-        ax = self.figure.add_subplot(1, 1, 1)
-
-        ax.clear()
+        self.ax.clear()
 
         # start
         if current_time <= self.interval / 2:
-            '''
-            Pxx, freqs, bins, im = ax.specgram(self.sound_info[: int((self.interval) * self.frame_rate)],
-                                               mode = 'psd' ,
-                                               NFFT=1024,
-                                               Fs=self.frame_rate,
-                                               noverlap=900,
-                                               vmin=VMIN)
-            '''
 
             time_ = np.linspace(0,
                                 len(self.sound_info[:int((self.interval) * self.frame_rate)]) / self.frame_rate,
                                 num=len(self.sound_info[:int((self.interval) * self.frame_rate)]))
-            ax.plot(time_, self.sound_info[:int((self.interval) * self.frame_rate)])
+            self.ax.plot(time_, self.sound_info[:int((self.interval) * self.frame_rate)])
 
-            ax.set_xlim(current_time - self.interval / 2, current_time + self.interval / 2)
-
-            #ax.set_ylim(Y_MIN, Y_MAX)
+            self.ax.set_xlim(current_time - self.interval / 2, current_time + self.interval / 2)
 
             # cursor
-            ax.axvline(x=current_time, color=self.cursor_color, linestyle="-")
+            self.ax.axvline(x=current_time, color=self.cursor_color, linestyle="-")
 
         elif current_time >= self.media_length - self.interval / 2:
 
             i = int(round(len(self.sound_info) - (self.interval * self.frame_rate), 0))
-            '''
-            Pxx, freqs, bins, im = ax.specgram(self.sound_info[i : ],
-                                               mode = "psd",
-                                               NFFT=1024,
-                                               Fs=self.frame_rate,
-                                               noverlap=900,
-                                               vmin=VMIN)
-            '''
 
             time_ = np.linspace(0, len(self.sound_info[i:]) / self.frame_rate, num=len(self.sound_info[i:]))
-            ax.plot(time_, self.sound_info[i:])
+            self.ax.plot(time_, self.sound_info[i:])
 
             lim1 = current_time - (self.media_length - self.interval / 2)
             lim2 = lim1 + self.interval
 
-            ax.set_xlim(lim1, lim2)
+            self.ax.set_xlim(lim1, lim2)
 
-            #ax.set_ylim(Y_MIN, Y_MAX)
-
-            ax.set_xticklabels([str(round(w + self.media_length - self.interval, 1)) for w in ax.get_xticks()])
+            self.ax.xaxis.set_major_locator(mticker.FixedLocator(self.ax.get_xticks().tolist()))
+            self.ax.set_xticklabels(
+                [str(round(w + self.media_length - self.interval, 1)) for w in self.ax.get_xticks()])
 
             # cursor
-            ax.axvline(x=lim1 + self.interval / 2, color=self.cursor_color, linestyle="-")
+            self.ax.axvline(x=lim1 + self.interval / 2, color=self.cursor_color, linestyle="-")
 
         # middle
         else:
 
             start = (current_time - self.interval / 2) * self.frame_rate
             end = (current_time + self.interval / 2) * self.frame_rate
-            '''
-            Pxx, freqs, bins, im = ax.specgram(self.sound_info[int(round((current_time - self.interval/2) * self.frame_rate, 0)):
-                                                               int(round((current_time + self.interval/2) * self.frame_rate, 0))],
-                                           mode = 'psd' ,
-                                           NFFT=1024,
-                                           Fs=self.frame_rate,
-                                           noverlap=900,
-                                               vmin=VMIN)
-            '''
-            time_ = np.linspace(
-                0,
-                len(self.sound_info[int(round(
-                    (current_time - self.interval / 2) *
-                    self.frame_rate, 0)):int(round((current_time + self.interval / 2) * self.frame_rate, 0))]) /
-                self.frame_rate,
-                num=len(self.sound_info[int(round(
-                    (current_time - self.interval / 2) *
-                    self.frame_rate, 0)):int(round((current_time + self.interval / 2) * self.frame_rate, 0))]))
-            ax.plot(
-                time_, self.sound_info[int(round(
-                    (current_time - self.interval / 2) *
-                    self.frame_rate, 0)):int(round((current_time + self.interval / 2) * self.frame_rate, 0))])
 
-            #ax.set_ylim(Y_MIN, Y_MAX)
+            time_ = np.linspace(0,
+                                len(self.sound_info[int(round(start, 0)):int(round(end, 0))]) / self.frame_rate,
+                                num=len(self.sound_info[int(round(start, 0)):int(round(end, 0))]))
 
-            ax.set_xticklabels([str(round(current_time + w - self.interval / 2, 1)) for w in ax.get_xticks()])
+            self.ax.plot(time_, self.sound_info[int(round(start, 0)):int(round(end, 0))])
+
+            self.ax.xaxis.set_major_locator(mticker.FixedLocator(self.ax.get_xticks().tolist()))
+            self.ax.set_xticklabels([str(round(current_time + w - self.interval / 2, 1)) for w in self.ax.get_xticks()])
 
             # cursor
-            ax.axvline(x=self.interval / 2, color=self.cursor_color, linestyle="-")
-
-        self.figure.subplots_adjust(wspace=0, hspace=0)
+            self.ax.axvline(x=self.interval / 2, color=self.cursor_color, linestyle="-")
+        '''self.figure.subplots_adjust(wspace=0, hspace=0)'''
 
         self.canvas.draw()
