@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 """
 BORIS
 Behavioral Observation Research Interactive Software
@@ -21,21 +20,21 @@ This file is part of BORIS.
 
 """
 
-from PyQt5.QtGui import *
+import re
+
 from PyQt5.QtCore import *
+from PyQt5.QtGui import *
 from PyQt5.QtWidgets import *
 
-import re
-from boris.config import *
-from boris.utilities import sorted_keys
+from . import config as cfg
+from . import utilities as util
 
 
 class ModifiersList(QDialog):
-
-    def __init__(self, code, modifiers_dict, currentModifier):
+    def __init__(self, code: str, modifiers_dict: dict, currentModifier: str):
 
         super().__init__()
-        self.setWindowTitle(programName)
+        self.setWindowTitle(cfg.programName)
         self.setWindowFlags(Qt.WindowStaysOnTopHint)
 
         self.modifiers_dict = dict(modifiers_dict)
@@ -49,9 +48,13 @@ class ModifiersList(QDialog):
         Hlayout = QHBoxLayout()
         self.modifiersSetNumber = 0
 
-        for idx in sorted_keys(modifiers_dict):
+        for idx in util.sorted_keys(modifiers_dict):
 
-            if self.modifiers_dict[idx]["type"] not in [SINGLE_SELECTION, MULTI_SELECTION, NUMERIC_MODIFIER]:
+            if self.modifiers_dict[idx]["type"] not in [
+                cfg.SINGLE_SELECTION,
+                cfg.MULTI_SELECTION,
+                cfg.NUMERIC_MODIFIER,
+            ]:
                 continue
 
             V2layout = QVBoxLayout()
@@ -62,42 +65,45 @@ class ModifiersList(QDialog):
             lb.setText(f"Modifier <b>{self.modifiers_dict[idx]['name']}</b>")
             V2layout.addWidget(lb)
 
-            if self.modifiers_dict[idx]["type"] in [SINGLE_SELECTION, MULTI_SELECTION]:
+            if self.modifiers_dict[idx]["type"] in [cfg.SINGLE_SELECTION, cfg.MULTI_SELECTION]:
                 lw = QListWidget()
                 self.modifiers_dict[idx]["widget"] = lw
                 lw.setObjectName(f"lw_modifiers_({self.modifiers_dict[idx]['type']})")
                 lw.installEventFilter(self)
 
-                if self.modifiers_dict[idx]["type"] == SINGLE_SELECTION:
+                if self.modifiers_dict[idx]["type"] == cfg.SINGLE_SELECTION:
                     item = QListWidgetItem("None")
                     lw.addItem(item)
                     item.setSelected(True)
 
                 for modifier in self.modifiers_dict[idx]["values"]:
                     item = QListWidgetItem(modifier)
-                    if self.modifiers_dict[idx]["type"] == MULTI_SELECTION:
+                    if self.modifiers_dict[idx]["type"] == cfg.MULTI_SELECTION:
                         item.setCheckState(Qt.Unchecked)
 
                         # previously selected
                         try:
-                            if currentModifierList != [""] and re.sub(
-                                    " \(.\)", "", modifier) in currentModifierList[int(idx)].split(","):
+                            if currentModifierList != [""] and re.sub(" \(.\)", "", modifier) in currentModifierList[
+                                int(idx)
+                            ].split(","):
                                 item.setCheckState(Qt.Checked)
                         except Exception:  # for old projects due to a fixed bug
                             pass
 
                     lw.addItem(item)
 
-                    if self.modifiers_dict[idx]["type"] == SINGLE_SELECTION:
+                    if self.modifiers_dict[idx]["type"] == cfg.SINGLE_SELECTION:
                         try:
-                            if currentModifierList != [""] and re.sub(" \(.\)", "",
-                                                                      modifier) == currentModifierList[int(idx)]:
+                            if (
+                                currentModifierList != [""]
+                                and re.sub(" \(.\)", "", modifier) == currentModifierList[int(idx)]
+                            ):
                                 item.setSelected(True)
                         except Exception:  # for old projects due to a fixed bug
                             pass
                 V2layout.addWidget(lw)
 
-            if self.modifiers_dict[idx]["type"] in [NUMERIC_MODIFIER]:
+            if self.modifiers_dict[idx]["type"] in [cfg.NUMERIC_MODIFIER]:
                 le = QLineEdit()
                 self.modifiers_dict[idx]["widget"] = le
 
@@ -117,11 +123,11 @@ class ModifiersList(QDialog):
         H2layout = QHBoxLayout()
         H2layout.addStretch(1)
 
-        pbCancel = QPushButton(CANCEL)
+        pbCancel = QPushButton(cfg.CANCEL)
         pbCancel.clicked.connect(self.reject)
         H2layout.addWidget(pbCancel)
 
-        pbOK = QPushButton(OK)
+        pbOK = QPushButton(cfg.OK)
         pbOK.setDefault(True)
         pbOK.clicked.connect(self.pbOK_clicked)
         H2layout.addWidget(pbOK)
@@ -136,7 +142,7 @@ class ModifiersList(QDialog):
         """
         send event (if keypress) to main window
         """
-        if (event.type() == QEvent.KeyPress):
+        if event.type() == QEvent.KeyPress:
             ek, ek_text = event.key(), event.text()
 
             # reject and close dialog if escape pressed
@@ -161,7 +167,8 @@ class ModifiersList(QDialog):
                             # modifiers have no associated code: the modifier starting with hit key will be selected
                             if ek not in [Qt.Key_Down, Qt.Key_Up]:
 
-                                if ek == Qt.Key_Space and f"({MULTI_SELECTION})" in widget.objectName(
+                                if (
+                                    ek == Qt.Key_Space and f"({cfg.MULTI_SELECTION})" in widget.objectName()
                                 ):  # checking using SPACE bar
                                     if widget.item(widget.currentRow()).checkState() == Qt.Checked:
                                         widget.item(widget.currentRow()).setCheckState(Qt.Unchecked)
@@ -185,16 +192,16 @@ class ModifiersList(QDialog):
 
                     for index in range(widget.count()):
 
-                        if ek in function_keys:
-                            if f"({function_keys[ek]})" in widget.item(index).text().upper():
-                                if f"({SINGLE_SELECTION})" in widget.objectName():
+                        if ek in cfg.function_keys:
+                            if f"({cfg.function_keys[ek]})" in widget.item(index).text().upper():
+                                if f"({cfg.SINGLE_SELECTION})" in widget.objectName():
                                     widget.item(index).setSelected(True)
                                     # close dialog if one set of modifiers
                                     if self.modifiersSetNumber == 1:
                                         self.accept()
                                         return True
 
-                                if f"({MULTI_SELECTION})" in widget.objectName():
+                                if f"({cfg.MULTI_SELECTION})" in widget.objectName():
                                     if widget.item(index).checkState() == Qt.Checked:
                                         widget.item(index).setCheckState(Qt.Unchecked)
                                     else:
@@ -202,14 +209,14 @@ class ModifiersList(QDialog):
 
                         if ek < 1114112 and f"({ek_text})" in widget.item(index).text():
 
-                            if f"({SINGLE_SELECTION})" in widget.objectName():
+                            if f"({cfg.SINGLE_SELECTION})" in widget.objectName():
                                 widget.item(index).setSelected(True)
                                 # close dialog if one set of modifiers
                                 if self.modifiersSetNumber == 1:
                                     self.accept()
                                     return True
 
-                            if f"({MULTI_SELECTION})" in widget.objectName():
+                            if f"({cfg.MULTI_SELECTION})" in widget.objectName():
                                 if widget.item(index).checkState() == Qt.Checked:
                                     widget.item(index).setCheckState(Qt.Unchecked)
                                 else:
@@ -225,28 +232,30 @@ class ModifiersList(QDialog):
         returns list of selected modifiers
         """
         modifiers = []
-        for idx in sorted_keys(self.modifiers_dict):
+        for idx in util.sorted_keys(self.modifiers_dict):
 
-            if self.modifiers_dict[idx]["type"] in [SINGLE_SELECTION, MULTI_SELECTION, NUMERIC_MODIFIER]:
+            if self.modifiers_dict[idx]["type"] in [cfg.SINGLE_SELECTION, cfg.MULTI_SELECTION, cfg.NUMERIC_MODIFIER]:
                 self.modifiers_dict[idx]["selected"] = []
 
-            if self.modifiers_dict[idx]["type"] == MULTI_SELECTION:
+            if self.modifiers_dict[idx]["type"] == cfg.MULTI_SELECTION:
                 for j in range(self.modifiers_dict[idx]["widget"].count()):
                     if self.modifiers_dict[idx]["widget"].item(j).checkState() == Qt.Checked:
                         self.modifiers_dict[idx]["selected"].append(
-                            re.sub(" \(.*\)", "", self.modifiers_dict[idx]["widget"].item(j).text()))
+                            re.sub(" \(.*\)", "", self.modifiers_dict[idx]["widget"].item(j).text())
+                        )
 
                 if not self.modifiers_dict[idx]["selected"]:
                     self.modifiers_dict[idx]["selected"].append("None")
 
-            if self.modifiers_dict[idx]["type"] == SINGLE_SELECTION:
+            if self.modifiers_dict[idx]["type"] == cfg.SINGLE_SELECTION:
                 for item in self.modifiers_dict[idx]["widget"].selectedItems():
                     self.modifiers_dict[idx]["selected"].append(re.sub(" \(.*\)", "", item.text()))
 
-            if self.modifiers_dict[idx]["type"] == NUMERIC_MODIFIER:
-                self.modifiers_dict[idx]["selected"] = self.modifiers_dict[idx]["widget"].text(
-                ) if self.modifiers_dict[idx]["widget"].text() else "None"
-        '''
+            if self.modifiers_dict[idx]["type"] == cfg.NUMERIC_MODIFIER:
+                self.modifiers_dict[idx]["selected"] = (
+                    self.modifiers_dict[idx]["widget"].text() if self.modifiers_dict[idx]["widget"].text() else "None"
+                )
+        """
         for widget in self.children():
             if widget.objectName() == "lw_modifiers_classic":
                 for item in widget.selectedItems():
@@ -255,21 +264,23 @@ class ModifiersList(QDialog):
                 for idx in range(widget.count()):
                     if widget.item(idx).checkState() == Qt.Checked:
                         modifiers.append(widget.item(idx).text())
-        '''
+        """
 
         return self.modifiers_dict
 
     def pbOK_clicked(self):
 
-        for idx in sorted_keys(self.modifiers_dict):
-            if self.modifiers_dict[idx]["type"] == NUMERIC_MODIFIER:
+        for idx in util.sorted_keys(self.modifiers_dict):
+            if self.modifiers_dict[idx]["type"] == cfg.NUMERIC_MODIFIER:
                 if self.modifiers_dict[idx]["widget"].text():
                     try:
                         val = float(self.modifiers_dict[idx]["widget"].text())
                     except Exception:
                         QMessageBox.warning(
-                            self, programName,
-                            "<b>{}</b> is not a numeric value".format(self.modifiers_dict[idx]["widget"].text()))
+                            self,
+                            cfg.programName,
+                            "<b>{}</b> is not a numeric value".format(self.modifiers_dict[idx]["widget"].text()),
+                        )
                         return
 
         self.accept()
